/*
 *  PowerPC simple emulation micro-operations for qemu.
 *  XXX: those micro-ops should be shared between all Qemu targets.
 * 
 *  Copyright (c) 2003-2005 Jocelyn Mayer
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

void op_cast32_T0 (void)
{
    T0 = (uint32_t)T0;
}

/***                           Sign extensions                             ***/
void op_exts8_T0 (void)
{
    T0 = (long)((int8_t)T0);
    RETURN();
}

void op_exts8_T1 (void)
{
    T1 = (long)((int8_t)T1);
    RETURN();
}

void op_exts16_T0 (void)
{
    T0 = (long)((int16_t)T0);
    RETURN();
}

void op_exts16_T1 (void)
{
    T1 = (long)((int16_t)T1);
    RETURN();
}

#if (HOST_LONG_BITS > TARGET_LONG_BITS) || (TARGET_LONG_BITS == 64)
void op_exts32_T0 (void)
{
    T0 = (long)((int32_t)T0);
    RETURN();
}

void op_exts32_T1 (void)
{
    T1 = (long)((int32_t)T1);
    RETURN();
}
#endif

/***                               Tn moves                                ***/
void op_move_T0_T1 (void)
{
    T0 = T1;
    RETURN();
}

void op_move_T0_T2 (void)
{
    T0 = T2;
    RETURN();
}

void op_move_T1_T0 (void)
{
    T1 = T0;
    RETURN();
}

void op_move_T1_T2 (void)
{
    T1 = T2;
    RETURN();
}

void op_move_T2_T0 (void)
{
    T2 = T0;
    RETURN();
}

void op_move_T2_T1 (void)
{
    T2 = T1;
    RETURN();
}

/***                            Constants load                             ***/
void op_reset_T0 (void)
{
    T0 = 0;
    RETURN();
}

void op_set_T0 (void)
{
    T0 = (long)(target_long)PARAM1;
    RETURN();
}

void op_reset_T1 (void)
{
    T1 = 0;
    RETURN();
}

void op_set_T1 (void)
{
    T1 = (long)(target_long)PARAM1;
    RETURN();
}

void op_reset_T2 (void)
{
    T2 = 0;
    RETURN();
}

void op_set_T2 (void)
{
    T2 = (long)(target_long)PARAM1;
    RETURN();
}

/***                            Integer logical                            ***/
void op_and_T0_T1 (void)
{
    T0 &= T1;
    RETURN();
}

void op_andi_T0 (void)
{
    T0 &= PARAM1;
    RETURN();
}

void op_andi_T1 (void)
{
    T1 &= PARAM1;
    RETURN();
}

#if TARGET_LONG_BITS == 64
void op_andi_T2 (void)
{
    T2 &= PARAM1;
    RETURN();
}
#endif

void op_andc_T0_T1 (void)
{
    T0 &= ~T1;
    RETURN();
}

void op_eqv_T0_T1 (void)
{
    T0 = ~(T0 ^ T1);
    RETURN();
}

void op_nand_T0_T1 (void)
{
    T0 = ~(T0 & T1);
    RETURN();
}

void op_nor_T0_T1 (void)
{
    T0 = ~(T0 | T1);
    RETURN();
}

void op_not_T0 (void)
{
    T0 = ~T0;
    RETURN();
}

void op_or_T0_T1 (void)
{
    T0 |= T1;
    RETURN();
}

void op_or_T0_T2 (void)
{
    T0 |= T2;
    RETURN();
}

void op_ori_T0 (void)
{
    T0 |= PARAM1;
    RETURN();
}

void op_orc_T0_T1 (void)
{
    T0 |= ~T1;
    RETURN();
}

void op_xor_T0_T1 (void)
{
    T0 ^= T1;
    RETURN();
}

void op_xori_T0 (void)
{
    T0 ^= PARAM1;
    RETURN();
}

/***                             Integer rotate                            ***/
void op_rotl8_T0_T1 (void)
{
    T0 = rotl8(T0, T1 & 0x07);
    RETURN();
}

void op_rotli8_T0 (void)
{
    T0 = rotl8(T0, PARAM1);
    RETURN();
}

void op_rotl16_T0_T1 (void)
{
    T0 = rotl16(T0, T1 & 0x0F);
    RETURN();
}

void op_rotli16_T0 (void)
{
    T0 = rotl16(T0, PARAM1);
    RETURN();
}

void op_rotl32_T0_T1 (void)
{
    T0 = rotl32(T0, T1 & 0x1F);
    RETURN();
}

void op_rotli32_T0 (void)
{
    T0 = rotl32(T0, PARAM1);
    RETURN();
}

#if TARGET_LONG_BITS == 64
void op_rotl64_T0_T1 (void)
{
    T0 = rotl64(T0, T1 & 0x3F);
    RETURN();
}

void op_rotli64_T0 (void)
{
    T0 = rotl64(T0, PARAM1);
    RETURN();
}
#endif

/***                             Integer shift                             ***/
void op_sl_T0_T1 (void)
{
    T0 = T0 << T1;
    RETURN();
}

void op_sli_T0 (void)
{
    T0 = T0 << PARAM1;
    RETURN();
}

void op_srl_T0_T1 (void)
{
    T0 = T0 >> T1;
    RETURN();
}

void op_srli_T0 (void)
{
    T0 = T0 >> PARAM1;
    RETURN();
}

void op_sra8_T0_T1 (void)
{
    T0 = (uint8_t)((int8_t)T0 >> T1);
    RETURN();
}

void op_srai8_T0 (void)
{
    T0 = (uint8_t)((int8_t)T0 >> PARAM1);
    RETURN();
}

void op_sra16_T0_T1 (void)
{
    T0 = (uint16_t)((int16_t)T0 >> T1);
    RETURN();
}

void op_srai16_T0 (void)
{
    T0 = (uint16_t)((int16_t)T0 >> PARAM1);
    RETURN();
}

void op_sra32_T0_T1 (void)
{
    T0 = (uint32_t)((int32_t)T0 >> T1);
    RETURN();
}

void op_srai32_T0 (void)
{
    T0 = (uint32_t)((int32_t)T0 >> PARAM1);
    RETURN();
}

#if TARGET_LONG_BITS == 64
void op_sra64_T0_T1 (void)
{
    T0 = (uint64_t)((int64_t)T0 >> T1);
    RETURN();
}

void op_srai64_T0 (void)
{
    T0 = (uint64_t)((int64_t)T0 >> PARAM1);
    RETURN();
}
#endif
